% Simulations for "On the Mis(Use) of Wealth as a Proxy for Risk Aversion,"
% by Marc F. Bellemare and Zachary S. Brown, American Journal of Agricultural Economics
% Code by Zachary S. Brown
% CARA Cases
% Feb 2009
% NOTE: This is a "light" (i.e., quickly converging) version of the simulations in the paper.
% Write to the author of the code at zsb2@duke.edu for a full version. See line 92 below.
clear; clc; tic;

% Utility functions and parameters
Rp = 1; % Principal's constant absolute risk aversion
Ra = 2; % Agent's constant absolute risk aversion
V = @(m) -(exp(-Rp.*m)); % Principal's v.N.M utility (over money)
U = @(m) -(exp(-Ra.*m)); % Agent's v.N.M utility (over money)
Uinv = @(U) -1.*log(-1.*U)./Ra;
phi = .005; % marginal disutility of agent's effort at e = 1
PHI = @(e) phi.*(e.^2)./2;
dPHIde = @(e) 2.*phi.*abs(e);  % positive

% Exogenous monetary variables
zp = 10; % principal's wealth
za =1.5; % agent's wealth
c = 0;  % Monetized value of agent's outside option 
Q1 = 1; % High output level
Q0 = 0; % low output level

% Probability "distributions"
chi = 1.5;  % "responsiveness" of distribution to agent's effort
P = @(e)  1-exp(-chi.*e); % Probability of high output Q1 given effort e
Pe = @(e) chi.*exp(-chi.*e); % Derivative of P w.r.t e  (>0)

% phi.*(chi.*exp(chi.*e) - chi) = DU.*chi.*exp(-chi.*e)
% 

% Agent's incentive compatibility constraint
% FOC: DU.*Pe = dPHIde; also note that this impliies a monotone increasing contract
nodisp = optimset('Display','Off');
FOC = @(DU,e) dPHIde(e) - DU.*Pe(e);
estar = @(DU) fminbnd( @(e) -1.*(DU.*P(e) - PHI(e)),0,1e6); 
% fsolve(@(e) FOC(DU,e),.5,nodisp);
% estar = @(DU) max([-log((phi./DU)./chi)./chi,zeros(size(DU))],[],2);  % optimal effort level as a function of state-contingent utility difference
% fplot(estar,[0,100])

% Agent's rationality constraint
RC = @(za,W1,W0,e) -(U(za + W1).*P(e) + U(za + W0).*(1-P(e)) - U(za + c) - PHI(e));
% fplot(@(W0) (RC(za,10,W0,estar(U(za + 10) - U(za + W0)))),[-za,10])
W0s = @(za,W1) fsolve( @(W0) 2000.*(RC(za,W1,W0,estar(U(za + W1) - U(za + W0)))),0,nodisp);  % Solving out the low contract amount
% fplot(@(W1) W0s(za,W1),[0 100])
% hold on
% fplot(@(W1) W0s(za+10,W1),[0 100],'-.or')

% test
% fplot(@(W1) W0S(za,W1),[-za,10])

% Principal's Objective
obj = @(zp,Q1,Q0,e,W1,W0)  V(zp + Q1 - W1).*P(e) + V(zp + Q0 - W0).*(1-P(e));

% Ws = @(za,zp,Q1,Q0) fmincon(@(W) obj(zp,Q1,Q0,estar(U(za + W(2)) - U(za + W(1)),W(2),W(1))),(Q0+Q1)./5.*ones(2,1),[],[],[],[],[],[],@(W) RC(za,W(2),W(1),estar(U(za + W(2)) - U(za + W(1)))));
% Ws(za,zp,Q1,Q0)

OBJ = @(za,zp,Q1,Q0,W1)   obj(zp,Q1,Q0,estar(U(za + W1) - U(za + W0s(za,W1))),W1,W0s(za,W1));  % substituting in constraints 
% fplot(@(W1) OBJ(za,zp,Q1,Q0,W1),[0 2])
W1s = @(za,zp,Q1,Q0) fminbnd(@(W1) -1.*OBJ(za,zp,Q1,Q0,W1),-za,100);

dWdQ = @(W1,za,zp,Q1,Q0) (W1 - W0s(za,W1))./(Q1-Q0); % Contract Slope / Risk Sharing

DWDQ = @(za,zp,Q1,Q0) dWdQ(W1s(za,zp,Q1,Q0),za,zp,Q1,Q0);
W1s(za,zp,Q1,Q0)
W0s(za,W1s(za,zp,Q1,Q0))
DWDQ(za,zp,Q1,Q0) 

RES = @(za,zp,Q1,Q0) [W1s(za,zp,Q1,Q0), W0s(za,W1s(za,zp,Q1,Q0)),DWDQ(za,zp,Q1,Q0),estar(U(za + W1s(za,zp,Q1,Q0)) - U(za + W0s(za,W1s(za,zp,Q1,Q0)))),RC(za,W1s(za,zp,Q1,Q0),W0s(za,W1s(za,zp,Q1,Q0)),estar(U(za + W1s(za,zp,Q1,Q0)) - U(za + W0s(za,W1s(za,zp,Q1,Q0)))))];

% fplot( @(za) DWDQ(za,zp,Q1,Q0), [0 5]);


%  Principal Isoutility curves given endogenized effort

xl = .1;
XL = .38;
OBJbase = OBJ(za,zp,Q1,Q0,W1s(za,zp,Q1,Q0)); 
OBJdel = OBJ(za.*1.5,zp,Q1,Q0,W1s(za.*1.5,zp,Q1,Q0)); 
isoOBJbase = @(W1) fsolve(@(W0) 2000.*(obj(zp,Q1,Q0,estar(U(za + W1) - U(za + W0)),W1,W0) - OBJbase),0,nodisp);
isoOBJdel = @(W1) fsolve( @(W0) 2000.*(obj(zp,Q1,Q0,estar(U(za.*1.5 + W1) - U(za.*1.5 + W0)),W1,W0) - OBJdel),0,nodisp);

% Counterfactual
COBJ = @(W1)   obj(zp,Q1,Q0,estar(U(za + W1) - U(za + W0s(1.5.*za,W1))),W1,W0s(1.5.*za,W1));
CW1s = fminbnd(@(W1) -1.*COBJ(W1),-za,100);
cobj = COBJ(CW1s);
isoCOBJ = @(W1) fsolve(@(W0) 2000.*(obj(zp,Q1,Q0,estar(U(za + W1) - U(za + W0)),W1,W0) - cobj),0,nodisp);

% Risk Sharing and Resids under Different agent wealth levels
bin = [];
N = 10;     % This controls the resolution of the risk-sharing v. agent wealth plot
            % In the paper, I used a much high resolution, maybe 100+ ...    

for r=0:N;
    
    disp(['Iteration: ',num2str(r)])
    zaS = 3.5.*(r./N); 
    W1 = W1s(zaS,zp,Q1,Q0);
    W0 = W0s(zaS,W1);
    DW = W1 - W0;
    Estar = estar(U(zaS + W1) - U(zaS + W0));
    resid = RC(zaS,W1,W0,Estar);
    bin = [bin; W1, W0, DW, Estar, resid, zaS];
    disp(num2str(bin(r+1,:)))  
    
    
end
x = bin(:,6);
y = bin(:,3);


% Need to install compecon toolbox on your matlab installation in order for
% commented out lines to function:
% fspace = fundefn('spli',2,0,3.5,2);
% Cfit= funfitxy(fspace,x,y);
% risksharing = @(za) funeval(Cfit,fspace,za);
figure(1)
plot(x,y,'*')
hold on
% fplot(risksharing,[0 3.5],'-r')
xlabel('Agent`s wealth (units of output)')
ylabel('Risk sharing (\Deltaw/\Deltaq)')
title('\bfRisk Sharing as a function of Agent Wealth: The CARA Case\rm')


figure(2)
fplot(@(W1) W0s(za,W1),[xl XL])
hold on
fplot(@(W1) W0s(za.*1.5,W1),[xl XL],'-r*')
hold on
fplot(@(W1) isoOBJbase(W1),[xl XL],':')
hold on
fplot(@(W1) isoOBJdel(W1),[xl XL],'-.r')
hold on 
fplot(@(W1) isoCOBJ(W1),[xl XL],'-.')
hold on 
title(['\bf Changing Agent Wealth under CARA: Constraints and Isoutility Curves (effort endogenous)\rm']);
hold on
plot(W1s(za,zp,Q1,Q0),W0s(za,W1s(za,zp,Q1,Q0)))
hold on
plot(W1s(za.*1.5,zp,Q1,Q0),W0s(za.*1.5,W1s(za.*1.5,zp,Q1,Q0)))
hold on 
plot(CW1s,W0s(1.5.*za,CW1s),'-.o')
xlabel('High Output Payment to Agent')
ylabel('Low Output Payment to Agent')
legend('Baseline RC Constraint','RC Constraint - Increase agent wealth by 50%','Optimal Principal Isoutility under baseline','Optimal Principal Isoutility following increase','Suboptimal Principal Isoutility Curve','Location','SouthWest')
ylim([-.8 0])

figure(3)
subplot(2,1,1)
plot(x,bin(:,1),'LineWidth',2)
ylabel('Optimal high output payment');
title('Optimal contract and effort as a function of agent wealth');

subplot(2,1,2)
plot(x,bin(:,4),'LineWidth',2)
ylabel('Optimal effort level');
xlabel('Agent`s wealth (units of output)')




% disp('OUTPUT')
% disp(['High contract payment       : ', num2str(W1s(za,zp,Q1,Q0))]);
% disp(['Low contract payment        : ', num2str(W0s(za,W1s(za,zp,Q1,Q0)))]);
% disp(['Optimal effort level        : ', num2str(estar(U(za + W1s(za,zp,Q1,Q0)) - U(za + W0s(za,W1s(za,zp,Q1,Q0)))))]);
% disp(['High output prob.           : ', num2str(P(estar(U(za + W1s(za,zp,Q1,Q0)) - U(za + W0s(za,W1s(za,zp,Q1,Q0))))))]);
% disp(['Expected output             : ', num2str(Q0 + (Q1 - Q0).*P(estar(U(za + W1s(za,zp,Q1,Q0)) - U(za + W0s(za,W1s(za,zp,Q1,Q0))))))]);
% disp(['Principal`s Expected utility: ', num2str(V(zp + Q0 - W0s(za,W1s(za,zp,Q1,Q0))) + (V(zp + Q1 - W1s(za,zp,Q1,Q0)) - V(zp + Q0 - W0s(za,W1s(za,zp,Q1,Q0)))).*P(estar(U(za + W1s(za,zp,Q1,Q0)) - U(za + W0s(za,W1s(za,zp,Q1,Q0))))))]);
% disp(['Rationality constraint      : ', num2str(RC(za,W1s(za,zp,Q1,Q0),W0s(za,W1s(za,zp,Q1,Q0)),estar(U(za + W1s(za,zp,Q1,Q0)) - U(za + W0s(za,W1s(za,zp,Q1,Q0))))))]);
% disp(['Contract Slope              : ', num2str(dWdQ(za,zp,Q1,Q0))]);

toc